@@ -18,32 +18,6 @@ class Module:
     force module. It's useful in situations when an upstream server
     needs to be targeted.
 
-    Arguments for the module that'll be passed along from the user are
-    defined in the function annotations for __init__. This means that
-    each parameter must receive an annotations.
-
-    ## Function Annotation Format
-
-    The function annotation format is a simple comma delimited list of
-    values that are passed along to argparse:
-    
-    required:(True|False),type:<Expected input type>,help:<Help string>
-
-    ### Example
-
-    The following example sets a static server IP address for a fake
-    module.
-
-    ```
-    def __init__(self,server_ip:'required:True,type:str,help:"ip of server"'):
-        pass
-    ```
-
-    ### Notes
-
-    - type is not actually passed to the type parameter of an argparse
-    argument. It's just there to be shown in the help menu.
-
     # The __call__ Method
 
     This method is called for each authentication attempt by BruteLoops
@@ -51,13 +25,13 @@ class Module:
     signature must look like:
 
     ```
-    def __call__(self, username, password):
+    def __call__(self, username, password, *args, **kwargs):
         success = False
 
         # Do authentication and update success to True if successful
 
-        if success: return (1,username,password,)
-        else: return (0,username,password,)
+        if success: return dict(outcome=1,username=username,password=password)
+        else: return dict(outcome=0,username=username,password=password,
     ```
 
     Note the structure returned in the declaration above. The leading
@@ -133,7 +107,7 @@ class Module:
     def get_handle(cls):
         '''Return a simple string to use as a module identifier.
         '''
-        return '.'.join(cls.__module__.split('.')[-2:])
+        return '.'.join(cls.__module__.split('.')[-3:][:2])
 
     @classmethod
     def build_interface(cls,
@@ -152,77 +126,57 @@ class Module:
         with providing a mechanism to assign values to module parameters.
         '''
 
+        epilog = None
+        if hasattr(cls, 'contributors'):
+            epilog = 'Module Contributors:\n\n'
+
+            if not isinstance(cls.contributors, list):
+
+                raise ValueError(
+                    'Module contributors must be a list of dictionary '
+                    f'values, not {type(cls.contributors)}')
+
+            for cont in cls.contributors:
+
+                if not isinstance(cont, dict):
+
+                    raise ValueError(
+                        'contributor records must be dictionaries, '
+                        f'not {type(cont)}')
+
+                name = cont.get('name')
+                additional = cont.get('additional')
+
+                if not name:
+
+                    raise ValueError(
+                        'contributor records must have a "name" field')
+
+                epilog += f'- {name}'
+
+                if additional:
+
+                    if not isinstance(additional, dict):
+
+                        raise ValueError(
+                            'additional field of contributor records '
+                            f'must be a dict, not {type(additional)}')
+
+                    for k,v in additional.items():
+
+                        epilog += f'\n  {k}: {v}'
+
         parser = subparsers.add_parser(cls.get_handle(),
                 description=cls.description,
-                help=cls.brief_description)
+                help=cls.brief_description,
+                parents=cls.args,
+                formatter_class=argparse.RawDescriptionHelpFormatter,
+                epilog=epilog)
+
         parser.set_defaults(module=cls)
 
-        # ========================
-        # HANDLE EMPTY ANNOTATIONS
-        # ========================
-        
-        annotations = cls.__init__.__annotations__
-        if not annotations: return parser
-
-        # ============================
-        # BUILD HELP FOR THE INTERFACE
-        # ============================
-        
-        # Regular expression string that will be used to parse the
-        # argument information from the function annotation
-        ARGPARSE_SIG = re.compile(
-            '^required:(?P<required>True|False),'
-            'type:(?P<type>.+?),'
-            '(nargs:(?P<nargs>.+),)?'
-            'help:(?P<help>.+)'
-        )
-        
-        # Signature to derive default values for the parameters
-        sig = inspect.signature(cls.__init__)
-
-        # ========================================
-        # BUILD ALL ARGUMENTS FROM THE ANNOTATIONS
-        # ========================================
-        
-        for arg, str in annotations.items():
-
-            # Parse the annotation
-            match = re.match(ARGPARSE_SIG, str)
-
-            # Throw an error when parsing fails
-            if not match:
-                raise ValueError(
-                        f'Error parsing help string for module {__file__}: ' \
-                        f'{str}')
-        
-            # Translate the match to a dictionary
-            dct = match.groupdict()
-        
-            # Convert the "required" element to a boolean
-            dct['required'] = True if dct['required'] == 'True' else False
-        
-            # Handle the default value
-            default = sig.parameters[arg].default
-
-            # Ensure the default isn't empty
-            default = default if default != \
-                    inspect.Parameter.empty else None
-
-            if default: dct['default']=default
-        
-            # Create the help string
-            dct['help']= ('required' if dct['required'] else 'optional') + \
-                f' - {dct["type"]} - {dct["help"]}'
-            del(dct['type'])
-
-            if 'default' in dct: dct['help']+= f' Default: {dct["default"]}'
-
-            # Handle nargs
-            if 'nargs' in dct and dct['nargs']:
-                # TODO: Update this to support other values
-                dct['nargs'] = '+'
-
-            # Add the argument to the parser
-            parser.add_argument(f'--{arg.replace("_","-")}', **dct)
+        parser.add_argument('--database', '-db',
+            required=True,
+            help='Database to target.')
 
         return parser
